% Simulation of LPG and VLSFO price, as a mean-reverting stochastic process


function [FuelDiff,VLSFO,LPG] = SimulateFuelPriceDifference(T_vessel,nSim,...
    LPG_start,VLSFO_start,Plots)

dt = 0.25;
sigma = 0;
kappa = 0.3;

%-------------------------
% LPG PRICE SIMUALATION
%-------------------------

St = log(LPG_start);
meanReverting = St;
OUmodel = hwv(kappa, meanReverting, sigma,  'StartState', St);
n_years_sim = T_vessel;
n_Steps = (1/dt)*n_years_sim;
[LPG, Time] = simulate(OUmodel,n_Steps,'nTrials',nSim, 'DeltaTime',dt);
LPG = exp(LPG);
LPG = squeeze(LPG);
LPG = LPG';

%----------------------------
%     VLSFO PRICE SIMULATION
%----------------------------
 
St = log(VLSFO_start);
meanReverting = St;
OUmodel = hwv(kappa, meanReverting, sigma,  'StartState', St);

n_years_sim = T_vessel;
n_Steps = (1/dt)*n_years_sim;
[VLSFO, Time] = simulate(OUmodel,n_Steps,'nTrials',nSim, 'DeltaTime',dt);
VLSFO = exp(VLSFO);
VLSFO = squeeze(VLSFO);
VLSFO = VLSFO';
Time = Time';
[r,c] = size(VLSFO);

%----------------------------------
% FUEL PRICE DIFFERENCE LPG vs VLSFO 
%---------------------------------
FuelDiff = zeros(r,c);
for i=1:r
    for j=1:c
        FuelDiff(i,j) = LPG(i,j) - VLSFO(i,j);
    end
end

Time_0 = 2020;
Time = Time_0 + Time;


if Plots
    for i=1:3
        figure(i)
        hold on
        plot(Time,VLSFO(i,:),'LineWidth',1.5)
        plot(Time,LPG(i,:),'LineWidth',1.5)
        plot(Time,FuelDiff(i,:),'LineWidth',1.5)
        legend('VLSFO','LPG','Price difference','Location','northwest')
        xlabel('Year')
        ylabel('Fuel Price [USD/tonne MGO eq.]')
    end
    histFuel = FuelDiff;
    histFuel(:,1) = [];
    histFuel = histFuel(:);
    maks = max(histFuel);
    minimum = min(histFuel);
    avg = mean(histFuel);
    mdn = median(histFuel);
    skew = skewness(histFuel);
    kurt = kurtosis(histFuel);
    prc10 = prctile(histFuel,10);
    prc90 = prctile(histFuel,90);

    figure
    histogram(histFuel,100)
    title('Fueldiff')
    xlim([-1000 1500])
    xlabel('Fuel price difference');
    ylabel('Frequency')
    line([mdn, mdn], ylim, 'LineWidth', 1.5, 'Color', 'r')
    line([avg, avg], ylim,'Color', 'g','LineWidth', 1.5)
    line([prc10, prc10], ylim, 'LineWidth', 1.5, 'Color', 'black')
    line([prc90, prc90], ylim, 'LineWidth', 1.5, 'Color', 'black')
end